import { isString } from './functions.js';

const inlineNodesRegexp = new RegExp([
    '(?<quote>^(?<quoteTag>> )(?<quoteContent>\\S+))',
    '(?<tag>#(?<tagContent>\\S+))',
    '(?<bold>\\[\\[(?<boldContent>.+)\\]\\])',
    '(?<deco>\\[(?<decoKeys>[*/_-]+?) (?<decoContent>.+?)\\])',
    '(?<decoFormula>\\[(?<decoFormulaContent>\\$ (?<formula>.+?))\\])',
    '(?<icon>\\[(\\/(?<iconProject>\\S+?)\\/)?(?<iconContent>(?<iconPage>\\S+)?\\.icon)\\])',
    '(?<code>`(?<codeContent>.*?)`)',
    '(?<link>\\[(?<linkContent>.+?)\\])',
    '(?<char>.)',
].join('|'), 'gu');
const parseInlineText = (text) => {
    if (text === '') {
        return '';
    }
    const nodes = [...text.matchAll(inlineNodesRegexp)]
        .map((match) => match.groups)
        .map((match) => {
        switch (true) {
            case typeof match.char === 'string': {
                return match.char;
            }
            case typeof match.quote === 'string': {
                return {
                    type: 'quote',
                    unit: {
                        content: match.quoteContent,
                        tag: match.quoteTag,
                        whole: match.quote,
                    },
                };
            }
            case typeof match.tag === 'string': {
                return {
                    type: 'hashTag',
                    unit: {
                        content: match.tagContent,
                        page: match.tagContent,
                        tag: '#',
                        whole: match.tag,
                    },
                };
            }
            case typeof match.bold === 'string': {
                return {
                    type: 'strong',
                    unit: {
                        content: match.boldContent,
                        whole: match.bold,
                    },
                };
            }
            case typeof match.deco === 'string': {
                return {
                    type: 'deco',
                    unit: {
                        content: match.decoContent,
                        deco: match.decoKeys,
                        italic: match.decoKeys.includes('/'),
                        strike: match.decoKeys.includes('-'),
                        underline: match.decoKeys.includes('_'),
                        strong: [...match.decoKeys].filter((char) => char === '*').length,
                        whole: match.deco,
                    },
                };
            }
            case typeof match.decoFormula === 'string': {
                return {
                    type: 'deco-formula',
                    unit: {
                        content: match.decoFormulaContent,
                        formula: match.formula,
                        whole: match.decoFormula,
                    },
                };
            }
            case typeof match.icon === 'string': {
                return {
                    type: 'icon',
                    unit: {
                        content: match.iconContent,
                        project: match.iconProject,
                        page: match.iconPage,
                        size: 1,
                        whole: match.icon,
                    },
                };
            }
            case typeof match.code === 'string': {
                return {
                    type: 'code',
                    unit: {
                        content: match.codeContent,
                        whole: match.code,
                    },
                };
            }
            case typeof match.link === 'string': {
                // TODO(feat): support urlLink
                return {
                    type: 'link',
                    unit: {
                        content: match.linkContent,
                        page: match.linkContent,
                        whole: match.link,
                    },
                };
            }
            default: {
                throw new Error('unreachable');
            }
        }
    })
        // concat chars to string
        .reduce((acc, node) => {
        const prev = acc.at(-1);
        if (isString(node) && isString(prev)) {
            acc.pop();
            acc.push(prev + node);
        }
        else {
            acc.push(node); // TODO(fix): correct type inference
        }
        return acc;
    }, []);
    return nodes.length === 1 ? nodes[0] : nodes;
};

export { parseInlineText };
//# sourceMappingURL=inline-nodes.js.map
